#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "lista.h"

PUNTO2D * p0;

// 1. Cuenta el número de puntos en el archivo "nombre"
// 2. Reserva la memoria para almacenar n puntos
// 3. Lee de nuevo el archivo para guardar los valores en el 
//      arreglo
LISTAPUNTO2D lee_puntos(char * nombre, int * pn)
{
	FILE * arch;
	char linea[80], *paux;
	int v1, v2, i;
	NODOPUNTO2D * point;
	LISTAPUNTO2D pts = NULL; //important to initialize the list

	arch = fopen( nombre, "r" );
	if( arch == NULL ) {
		fprintf( stderr, "No se puede abrir el archivo %s\n", nombre );
		return NULL;
	}

	*pn = 0;
	while( fgets( linea, 80, arch ) != NULL ) {
		paux = linea;
		while ( *paux == ' ' || *paux == '\t' )
			paux++;
		i = sscanf( paux, "%d %d\n", &v1, &v2 );
		if ( i == 2 ) {
			point = crea_elemento();
			point->x = v1;
			point->y = v2;
			inserta_elemento(&pts, point);
			(*pn)++;
		}
	}
	return pts;
}

int compara(const void *p1, const void *p2)
{
	PUNTO2D *pa, *pb;
	PUNTO2D v1, v2;
	int area2, d1, d2;

	pa = (PUNTO2D *)p1;
	pb = (PUNTO2D *)p2;

	v1.x = pa->x - p0->x;
	v1.y = pa->y - p0->y;

	v2.x = pb->x - p0->x;
	v2.y = pb->y - p0->y;

	area2 = v1.x * v2.y - v1.y * v2.x;

	if( area2 > 0 ) return -1;
	else if (area2 < 0 ) return 1;
	else {
		d1 = v1.x * v1.x + v1.y * v1.y;
		d2 = v2.x * v2.x + v2.y * v2.y;

		if ( d1 < d2 ) return -1;
		else return 1;
	}
}

void ordenamientoCircular(LISTAPUNTO2D ptos, int n)
{
	int i, minimo;
	PUNTO2D point_aux;
	PUNTO2D * point_array;
	LISTAPUNTO2D ptos_it = ptos->psig; //this list starts from the second node

	// Busco el punto con y mínima, si hay varios de ellos
	// encuentro el punto con x mínima:
	NODOPUNTO2D *min = ptos; //This is the first node

	while( ptos_it != NULL) {
		if(ptos_it->y < min->y)
			min = ptos_it;
		else if (ptos_it->y == min->y && ptos_it->x < min->x)
			min = ptos_it;
		ptos_it = ptos_it -> psig;
	}

	//Swaps values between the node keeping the minimum with the front node
	point_aux.x = min->x;
	point_aux.y = min->y;
	min->x = ptos->x;
	min->y = ptos->y;
	ptos->x = point_aux.x;
	ptos->y = point_aux.y;

	point_array = list_to_array(ptos, n);

	p0 = point_array;
	//printf( "MIN: %d %d\n", p0[0].x, p0[0].y );

	// Ordenamos los puntos!
	qsort((void *)&point_array[1], n-1, sizeof(PUNTO2D), compara);
	array_to_list(point_array, &ptos, n);
	free(point_array);
}

// int main(int argc, char *argv[])
// {
//     PUNTO2D *puntos;
// 	int i, n;
// 	char nombre[32];

// 	if( argc != 2 ) {
// 		fprintf( stderr, "Args: archivo_entrada\n" );
// 		return 1;
// 	}
// 	strncpy(nombre, argv[1], 32);

// 	puntos = lee_puntos( nombre, &n );

// 	if ( puntos != NULL ) {
// 		ordenamientoCircular( puntos, n );

// 		for(i=0; i<n; i++) 
// 			printf( "%d %d\n", puntos[i].x, puntos[i].y );

// 		free( puntos );
// 		printf( "# Ok!\n" );
// 	}
// 	return 0;
// }

int main(int argc, char* argv[]){
	LISTAPUNTO2D point_list;
	int point_number;
	if( argc != 2 ) {
		fprintf( stderr, "Args: archivo_entrada\n" );
		return 1;
	}
	point_list = lee_puntos(argv[1], &point_number);
	if ( point_list != NULL ) {
		ordenamientoCircular( point_list, point_number );
		imprime_lista(point_list);
		borra_lista(&point_list);
		printf( "# Ok!\n" );
	}
	return 0;
}